require 'luda/version'
require 'luda/config'

module Luda
  # Hand copied and modified from Bootstrap gem
  # https://github.com/twbs/bootstrap-rubygem/blob/master/lib/bootstrap.rb
  class << self
    def load!
      if rails?
        register_rails_engine
      elsif hanami?
        register_hanami
      elsif sprockets?
        register_sprockets
      elsif defined?(::SassC) && ::SassC.respond_to?(:load_paths)
        ::SassC.load_paths << stylesheets_path
      elsif defined?(::Sass) && ::Sass.respond_to?(:load_paths)
        # The deprecated `sass` gem:
        ::Sass.load_paths << stylesheets_path
      end

      if defined?(::SassC::Script::Value::Number)
        ::SassC::Script::Value::Number.precision = [Luda::SASS_PRECISION, ::SassC::Script::Value::Number.precision].max
      end

      if defined?(::Sass::Script::Value::Number)
        ::Sass::Script::Value::Number.precision = [Luda::SASS_PRECISION, ::Sass::Script::Value::Number.precision].max
      end
    end

    # Paths
    def gem_path
      @gem_path ||= File.expand_path '..', File.dirname(__FILE__)
    end

    def stylesheets_path
      File.join assets_path, 'stylesheets'
    end

    def javascripts_path
      File.join assets_path, 'javascripts'
    end

    def assets_path
      @assets_path ||= File.join gem_path, 'assets'
    end

    # Environment detection helpers
    def sprockets?
      defined?(::Sprockets)
    end

    def rails?
      defined?(::Rails)
    end

    def hanami?
      defined?(::Hanami)
    end

    private

    def register_rails_engine
      require 'luda/engine'
    end

    def register_sprockets
      Sprockets.append_path(stylesheets_path)
      Sprockets.append_path(javascripts_path)
    end

    def register_hanami
      Hanami::Assets.sources << assets_path
    end
  end
end

Luda.load!
